@@ -66,44 +66,44 @@ def read_kafka(spark_session: SparkSession, source_conf: dict) -> DataFrame:
 
 
 def write_kafka_stream(stream: DataFrame, spark_job_conf: dict, output_name):
-    output_conf = get_spark_conf(output_name, spark_job_conf['outputs'])
+    output_stream_conf = get_spark_conf(output_name, spark_job_conf['outputs'])
 
-    stream = serialize_register_avro_stream(stream, output_conf['kafka_serialization'])
+    stream = serialize_register_avro_stream(stream, output_stream_conf['kafka_serialization'])
 
-    if output_conf.get('processing_time') is not None:
+    if output_stream_conf.get('processing_time') is not None:
         stream \
             .writeStream \
             .format("kafka") \
-            .options(**output_conf['kafka_options']) \
-            .trigger(processingTime=output_conf['processing_time']) \
-            .outputMode("append") \
+            .options(**output_stream_conf['kafka_options']) \
+            .trigger(processingTime=output_stream_conf['processing_time']) \
+            .outputMode(output_stream_conf['mode']) \
             .start() \
             .awaitTermination()
-    elif output_conf.get('availableNow') is not None:
+    elif output_stream_conf.get('availableNow') is not None:
         stream \
             .writeStream \
             .format("kafka") \
-            .options(**output_conf['kafka_options']) \
-            .trigger(availableNow=output_conf['availableNow']) \
-            .outputMode("append") \
+            .options(**output_stream_conf['kafka_options']) \
+            .trigger(availableNow=output_stream_conf['availableNow']) \
+            .outputMode(output_stream_conf['mode']) \
             .start() \
             .awaitTermination()
-    elif output_conf.get("once") is not None:
-        print(f'You choose to write into kafka stream with trigger.once={output_conf.get("once")}')
+    elif output_stream_conf.get("once") is not None:
+        print(f'You choose to write into kafka stream with trigger.once={output_stream_conf.get("once")}')
         stream \
             .writeStream \
             .format("kafka") \
-            .options(**output_conf['kafka_options']) \
-            .trigger(once=output_conf['once']) \
-            .outputMode("append") \
+            .options(**output_stream_conf['kafka_options']) \
+            .trigger(once=output_stream_conf['once']) \
+            .outputMode(output_stream_conf['mode']) \
             .start() \
             .awaitTermination()
     else:
         stream \
             .writeStream \
             .format("kafka") \
-            .options(**output_conf['kafka_options']) \
-            .outputMode("append") \
+            .options(**output_stream_conf['kafka_options']) \
+            .outputMode(output_stream_conf['mode']) \
             .start() \
             .awaitTermination()
 
@@ -111,11 +111,6 @@ def write_kafka_stream(stream: DataFrame, spark_job_conf: dict, output_name):
 ######
 # S3 #
 ######
-def read_s3_xxx(spark_session: SparkSession, spark_job_conf: dict, source_path: str):
-    source_conf = get_spark_conf(source_path, spark_job_conf['sources'])
-    raise NotImplementedError
-
-
 def read_s3_compress_log_stream(spark_session: SparkSession, spark_job_conf: dict, source_name: str) -> DataFrame:
     source_stream_conf = get_spark_conf(source_name, spark_job_conf['sources'])
     return spark_session \
@@ -197,80 +192,127 @@ def write_delta_stream(spark_job_conf: dict, output_name: str, stream: DataFrame
     if output_stream_conf.get('processing_time') is not None:
         stream \
             .writeStream \
+            .queryName(output_stream_conf['name']) \
             .format("delta") \
-            .outputMode("append") \
+            .outputMode(output_stream_conf['mode']) \
             .trigger(processingTime=output_stream_conf['processing_time']) \
             .partitionBy(output_stream_conf['delta_options']['partition_columns']) \
             .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
-            .start(output_stream_conf['delta_options']['location']) \
-            .awaitTermination()
+            .start(output_stream_conf['delta_options']['location'])
     elif output_stream_conf.get('availableNow') is not None:
         stream \
             .writeStream \
+            .queryName(output_stream_conf['name']) \
             .format("delta") \
-            .outputMode("append") \
+            .outputMode(output_stream_conf['mode']) \
             .trigger(availableNow=output_stream_conf.get('availableNow')) \
             .partitionBy(output_stream_conf['delta_options']['partition_columns']) \
             .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
-            .start(output_stream_conf['delta_options']['location']) \
-            .awaitTermination()
+            .start(output_stream_conf['delta_options']['location'])
     elif output_stream_conf.get('once') is not None:
-        print(f'You choose to write into delta stream with trigger.once={output_stream_conf.get("once")}')
         stream \
             .writeStream \
+            .queryName(output_stream_conf['name']) \
             .format("delta") \
-            .outputMode("append") \
+            .outputMode(output_stream_conf['mode']) \
             .trigger(once=output_stream_conf.get('once')) \
             .partitionBy(output_stream_conf['delta_options']['partition_columns']) \
             .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
-            .start(output_stream_conf['delta_options']['location']) \
-            .awaitTermination()
+            .start(output_stream_conf['delta_options']['location'])
     else:
         stream \
             .writeStream \
+            .queryName(output_stream_conf['name']) \
             .format("delta") \
-            .outputMode("append") \
+            .outputMode(output_stream_conf['mode']) \
             .partitionBy(output_stream_conf['delta_options']['partition_columns']) \
             .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
-            .start(output_stream_conf['delta_options']['location']) \
-            .awaitTermination()
+            .start(output_stream_conf['delta_options']['location'])
 
 
 def write_delta_stream_no_partitioning(spark_job_conf: dict, output_name: str, stream: DataFrame):
     output_stream_conf = get_spark_conf(output_name, spark_job_conf['outputs'])
     create_delta_tables_if_not_exists_no_partitioning(output_stream_conf, output_stream_conf['delta_options']['schema'])
-    stream \
-        .writeStream \
-        .format("delta") \
-        .outputMode("append") \
-        .trigger(availableNow=True) \
-        .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
-        .start(output_stream_conf['delta_options']['location'])
+
+    if output_stream_conf.get('processing_time') is not None:
+        stream \
+            .writeStream \
+            .queryName(output_stream_conf['name']) \
+            .format("delta") \
+            .outputMode(output_stream_conf['mode']) \
+            .trigger(processingTime=output_stream_conf['processing_time']) \
+            .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
+            .start(output_stream_conf['delta_options']['location'])
+    elif output_stream_conf.get('availableNow') is not None:
+        stream \
+            .writeStream \
+            .queryName(output_stream_conf['name']) \
+            .format("delta") \
+            .outputMode(output_stream_conf['mode']) \
+            .trigger(availableNow=output_stream_conf.get('availableNow')) \
+            .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
+            .start(output_stream_conf['delta_options']['location'])
+    elif output_stream_conf.get('once') is not None:
+        stream \
+            .writeStream \
+            .queryName(output_stream_conf['name']) \
+            .format("delta") \
+            .outputMode(output_stream_conf['mode']) \
+            .trigger(once=output_stream_conf.get('once')) \
+            .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
+            .start(output_stream_conf['delta_options']['location'])
+    else:
+        stream \
+            .writeStream \
+            .queryName(output_stream_conf['name']) \
+            .format("delta") \
+            .outputMode(output_stream_conf['mode']) \
+            .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
+            .start(output_stream_conf['delta_options']['location'])
 
 
 def write_delta_stream_with_custom_fnc(spark_job_conf, output_name: str, stream: DataFrame, updater_function):
     output_stream_conf = get_spark_conf(output_name, spark_job_conf['outputs'])
-    stream \
-        .writeStream \
-        .format("delta") \
-        .outputMode("update") \
-        .trigger(processingTime=output_stream_conf['delta_options']['processing_time']) \
-        .foreachBatch(updater_function) \
-        .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
-        .start(output_stream_conf['delta_options']['location'])
 
-
-def write_delta_stream_with_custom_fnc_await(spark_job_conf, output_name: str, stream: DataFrame, updater_function):
-    output_stream_conf = get_spark_conf(output_name, spark_job_conf['outputs'])
-    stream \
-        .writeStream \
-        .format("delta") \
-        .outputMode("update") \
-        .trigger(processingTime=output_stream_conf['delta_options']['processing_time']) \
-        .foreachBatch(updater_function) \
-        .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
-        .start(output_stream_conf['delta_options']['location']) \
-        .awaitTermination()
+    if output_stream_conf.get('processing_time') is not None:
+        stream \
+            .writeStream \
+            .queryName(output_stream_conf['name']) \
+            .format("delta") \
+            .outputMode(output_stream_conf['mode']) \
+            .trigger(processingTime=output_stream_conf['processing_time']) \
+            .foreachBatch(updater_function) \
+            .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
+            .start(output_stream_conf['delta_options']['location'])
+    elif output_stream_conf.get('availableNow') is not None:
+        stream \
+            .writeStream \
+            .queryName(output_stream_conf['name']) \
+            .format("delta") \
+            .outputMode(output_stream_conf['mode']) \
+            .trigger(availableNow=output_stream_conf['availableNow']) \
+            .foreachBatch(updater_function) \
+            .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
+            .start(output_stream_conf['delta_options']['location'])
+    elif output_stream_conf.get('once') is not None:
+        stream \
+            .writeStream \
+            .queryName(output_stream_conf['name']) \
+            .format("delta") \
+            .outputMode(output_stream_conf['mode']) \
+            .trigger(once=output_stream_conf['once']) \
+            .foreachBatch(updater_function) \
+            .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
+            .start(output_stream_conf['delta_options']['location'])
+    else:
+        stream \
+            .writeStream \
+            .queryName(output_stream_conf['name']) \
+            .format("delta") \
+            .outputMode(output_stream_conf['mode']) \
+            .foreachBatch(updater_function) \
+            .option("checkpointLocation", output_stream_conf['delta_options']['checkpoint_location']) \
+            .start(output_stream_conf['delta_options']['location'])
 
 
 ########
